% control_lsg7.m
% Aufgabe Kap. 5.6.7
% Regelung der instabilen PT$_2$-bertragungsfunktion
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 1. Instabiles System (c=-1)

% Strecke definieren
a = 1 ; b = 0.5 ; c = -1 ; K = 1 ;
sys = ss([ -b/a -c/a ; 1 0 ],[K/a ; 0],[0 1],0) 

% P-Regler: -c/K < Vp =>  stabil
reg = tf([-c/K+10*abs(c/K)],[1])                % Regler-Koeffizient
sys_r = feedback(reg*tf(sys),1);                % Geregeltes System

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 2. Instabiles System (c=-1)

% Zustands-Regler mit Sollwertanpassung
K  = place(sys.a,sys.b,[-1.1 -1]')              % Regler-Rckfhrmatrix
Kv = -1 / (sys.c*inv(sys.a-sys.b*K)*sys.b)      % Sollwertanpassung
sys_z = ss(sys.a-sys.b*K,sys.b,sys.c,0);        % Geregeltes System
p_z = pole(sys_z)

figure
    step(sys_r,'g-.',sys_z,'b--',sys_z*Kv,'b-')
    legend('P-Regler','Zustands-Regler','Sollwertanpassung',4)

if exist('druck') == 1 , print('-depsc','lsg_instabilPZ.eps') , end
if exist('druck') == 1 , print('-deps' ,'lsg_instabilPZ_sw.eps') , end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 3. Instabiles System (b=-0.5) mit Fhrungsintegrator und Strung 

% Strecke definieren
a = 1 ; b = -0.5 ; c = 1 ; K = 1 ;
sys = ss([ -b/a -c/a ; 1 0 ],[K/a ; 0],[0 1],0) 

% Zustands-Regler mit Sollwertanpassung
K  = place(sys.a,sys.b,[-1.1 -1]')              % Regler-Rckfhrmatrix
Kv = -1 / (sys.c*inv(sys.a-sys.b*K)*sys.b)      % Sollwertanpassung
sys_s = ss(sys.a-sys.b*K,[sys.b [2;0]],sys.c,0)*Kv; 

% Zustands-Regler mit Fhrungsintegrator
sys_f = ss([sys.a [0;0] ; -sys.c 0],[sys.b ; 0],[sys.c 0],0)
Kf = place(sys_f.a,sys_f.b,[-1.1 -1 -0.9])      % Regler-Rckfhrmatrix
sys_zf = ss(sys_f.a-sys_f.b*Kf,[0 2 ; 0 0 ; 1 0],[sys.c 0],0)

% Zeit und Eingangssignale erzeugen
t = 0:0.01:50 ;
w = t > 0 ;                                     % Fhrungs-Sprung bei t=1
z = 0.3*(t > 15) - 0.5*(t > 35) ;               % Str-Sprung bei t=15
y_s  = lsim(sys_s,[w' z'],t) ;                  % Sollwertanpassung
y_zf = lsim(sys_zf,[w' z'],t) ;                 % Fhrungsintegrator

figure
    hold on; grid on
    plot(t,z,'k-.',t,y_s,'b--',t,y_zf,'r-')
    ax=axis; axis([ax(1) ax(2) -0.3 1.7 ])
    legend('Strung','Sollwertanpassung','Fhrungsintegrator',2)
    title('Fhrungs- und Strverhalten')
    xlabel('Time (sec)'); ylabel('Amplitude')
    
if exist('druck') == 1 , print('-depsc','lsg_instabilZF.eps') , end
if exist('druck') == 1 , print('-deps' ,'lsg_instabilZF_sw.eps') , end
